pub fn create_test_device_settings() -> Result<DeviceSettings>Expand description
Create a test-specific DeviceSettings with memory storage
This creates a minimal configuration for testing purposes that uses only in-memory storage.
pub fn create_test_device_settings() -> Result<DeviceSettings>Create a test-specific DeviceSettings with memory storage
This creates a minimal configuration for testing purposes that uses only in-memory storage.